#
# Copyright 2022 The AI Flow Authors
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied.  See the License for the
# specific language governing permissions and limitations
# under the License.
#


# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: notification_service.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
from google.protobuf import service as _service
from google.protobuf import service_reflection
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.api import annotations_pb2 as google_dot_api_dot_annotations__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='notification_service.proto',
  package='notification_service',
  syntax='proto3',
  serialized_options=b'\n\035org.aiflow.notification.protoZ\025/notification_service\210\001\001\220\001\001',
  create_key=_descriptor._internal_create_key,
  serialized_pb=b'\n\x1anotification_service.proto\x12\x14notification_service\x1a\x1cgoogle/api/annotations.proto\"\x81\x01\n\nEventProto\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t\x12\x0f\n\x07\x63ontext\x18\x03 \x01(\t\x12\x11\n\tnamespace\x18\x04 \x01(\t\x12\x0e\n\x06offset\x18\x05 \x01(\x03\x12\x13\n\x0b\x63reate_time\x18\x06 \x01(\x03\x12\x0e\n\x06sender\x18\x07 \x01(\t\"<\n\x15SenderEventCountProto\x12\x0e\n\x06sender\x18\x01 \x01(\t\x12\x13\n\x0b\x65vent_count\x18\x02 \x01(\x03\"m\n\x10SendEventRequest\x12/\n\x05\x65vent\x18\x01 \x01(\x0b\x32 .notification_service.EventProto\x12\x0c\n\x04uuid\x18\x02 \x01(\t\x12\x1a\n\x12\x65nable_idempotence\x18\x03 \x01(\x08\"/\n\nClientMeta\x12\x11\n\tnamespace\x18\x01 \x01(\t\x12\x0e\n\x06sender\x18\x02 \x01(\t\"\x92\x01\n\x12SendEventsResponse\x12/\n\x05\x65vent\x18\x01 \x01(\x0b\x32 .notification_service.EventProto\x12\x37\n\x0breturn_code\x18\x02 \x01(\x0e\x32\".notification_service.ReturnStatus\x12\x12\n\nreturn_msg\x18\x03 \x01(\t\"\x86\x01\n\x11ListEventsRequest\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\x11\n\tnamespace\x18\x02 \x01(\t\x12\x0e\n\x06sender\x18\x03 \x01(\t\x12\x14\n\x0cstart_offset\x18\x04 \x01(\x03\x12\x12\n\nend_offset\x18\x05 \x01(\x03\x12\x17\n\x0ftimeout_seconds\x18\x06 \x01(\x05\"n\n\x12\x43ountEventsRequest\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\x11\n\tnamespace\x18\x02 \x01(\t\x12\x0e\n\x06sender\x18\x03 \x01(\t\x12\x14\n\x0cstart_offset\x18\x04 \x01(\x03\x12\x12\n\nend_offset\x18\x05 \x01(\x03\"m\n\x14ListAllEventsRequest\x12\x17\n\x0ftimeout_seconds\x18\x01 \x01(\x05\x12\x12\n\nstart_time\x18\x02 \x01(\x03\x12\x14\n\x0cstart_offset\x18\x03 \x01(\x03\x12\x12\n\nend_offset\x18\x04 \x01(\x03\"\x93\x01\n\x12ListEventsResponse\x12\x37\n\x0breturn_code\x18\x01 \x01(\x0e\x32\".notification_service.ReturnStatus\x12\x12\n\nreturn_msg\x18\x02 \x01(\t\x12\x30\n\x06\x65vents\x18\x03 \x03(\x0b\x32 .notification_service.EventProto\"\xc1\x01\n\x13\x43ountEventsResponse\x12\x37\n\x0breturn_code\x18\x01 \x01(\x0e\x32\".notification_service.ReturnStatus\x12\x12\n\nreturn_msg\x18\x02 \x01(\t\x12\x13\n\x0b\x65vent_count\x18\x03 \x01(\x03\x12H\n\x13sender_event_counts\x18\x04 \x03(\x0b\x32+.notification_service.SenderEventCountProto\"G\n\x0bMemberProto\x12\x0f\n\x07version\x18\x01 \x01(\x03\x12\x12\n\nserver_uri\x18\x02 \x01(\t\x12\x13\n\x0bupdate_time\x18\x03 \x01(\x03\"(\n\x06Notify\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\x11\n\tnamespace\x18\x02 \x01(\t\"?\n\rNotifyRequest\x12.\n\x08notifies\x18\x01 \x03(\x0b\x32\x1c.notification_service.Notify\"-\n\x12ListMembersRequest\x12\x17\n\x0ftimeout_seconds\x18\x01 \x01(\x05\"\x96\x01\n\x13ListMembersResponse\x12\x37\n\x0breturn_code\x18\x01 \x01(\x0e\x32\".notification_service.ReturnStatus\x12\x12\n\nreturn_msg\x18\x02 \x01(\t\x12\x32\n\x07members\x18\x03 \x03(\x0b\x32!.notification_service.MemberProto\"K\n\x16NotifyNewMemberRequest\x12\x31\n\x06member\x18\x01 \x01(\x0b\x32!.notification_service.MemberProto\"N\n\x15RegisterClientRequest\x12\x35\n\x0b\x63lient_meta\x18\x01 \x01(\x0b\x32 .notification_service.ClientMeta\"x\n\x16RegisterClientResponse\x12\x37\n\x0breturn_code\x18\x01 \x01(\x0e\x32\".notification_service.ReturnStatus\x12\x12\n\nreturn_msg\x18\x02 \x01(\t\x12\x11\n\tclient_id\x18\x03 \x01(\x03\"$\n\x0f\x43lientIdRequest\x12\x11\n\tclient_id\x18\x01 \x01(\x03\"]\n\x0e\x43ommonResponse\x12\x37\n\x0breturn_code\x18\x01 \x01(\x0e\x32\".notification_service.ReturnStatus\x12\x12\n\nreturn_msg\x18\x02 \x01(\t\"x\n\x16isClientExistsResponse\x12\x37\n\x0breturn_code\x18\x01 \x01(\x0e\x32\".notification_service.ReturnStatus\x12\x12\n\nreturn_msg\x18\x02 \x01(\t\x12\x11\n\tis_exists\x18\x03 \x01(\x08\"(\n\x13TimeToOffsetRequest\x12\x11\n\ttimestamp\x18\x01 \x01(\x03\"_\n\x14TimeToOffsetResponse\x12\x37\n\x0breturn_code\x18\x01 \x01(\x0e\x32\".notification_service.ReturnStatus\x12\x0e\n\x06offset\x18\x02 \x01(\x03*&\n\x0cReturnStatus\x12\x0b\n\x07SUCCESS\x10\x00\x12\t\n\x05\x45RROR\x10\x01\x32\xc7\r\n\x13NotificationService\x12\x91\x01\n\tsendEvent\x12&.notification_service.SendEventRequest\x1a(.notification_service.SendEventsResponse\"2\x82\xd3\xe4\x93\x02,\"\'/aiflow/notification-service/send_event:\x01*\x12\x91\x01\n\nlistEvents\x12\'.notification_service.ListEventsRequest\x1a(.notification_service.ListEventsResponse\"0\x82\xd3\xe4\x93\x02*\x12(/aiflow/notification-service/list_events\x12\x95\x01\n\x0b\x63ountEvents\x12(.notification_service.CountEventsRequest\x1a).notification_service.CountEventsResponse\"1\x82\xd3\xe4\x93\x02+\x12)/aiflow/notification-service/count_events\x12\x9b\x01\n\rlistAllEvents\x12*.notification_service.ListAllEventsRequest\x1a(.notification_service.ListEventsResponse\"4\x82\xd3\xe4\x93\x02.\x12,/aiflow/notification-service/list_all_events\x12\x83\x01\n\x06notify\x12#.notification_service.NotifyRequest\x1a$.notification_service.CommonResponse\".\x82\xd3\xe4\x93\x02(\"#/aiflow/notification-service/notify:\x01*\x12\x95\x01\n\x0blistMembers\x12(.notification_service.ListMembersRequest\x1a).notification_service.ListMembersResponse\"1\x82\xd3\xe4\x93\x02+\x12)/aiflow/notification-service/list_members\x12\xa0\x01\n\x0fnotifyNewMember\x12,.notification_service.NotifyNewMemberRequest\x1a$.notification_service.CommonResponse\"9\x82\xd3\xe4\x93\x02\x33\"./aiflow/notification-service/notify_new_member:\x01*\x12\xa4\x01\n\x0eregisterClient\x12+.notification_service.RegisterClientRequest\x1a,.notification_service.RegisterClientResponse\"7\x82\xd3\xe4\x93\x02\x31\",/aiflow/notification-service/register_client:\x01*\x12\x92\x01\n\x0c\x64\x65leteClient\x12%.notification_service.ClientIdRequest\x1a$.notification_service.CommonResponse\"5\x82\xd3\xe4\x93\x02/\"*/aiflow/notification-service/delete_client:\x01*\x12\x9f\x01\n\x0eisClientExists\x12%.notification_service.ClientIdRequest\x1a,.notification_service.isClientExistsResponse\"8\x82\xd3\xe4\x93\x02\x32\"-/aiflow/notification-service/is_client_exists:\x01*\x12\xb2\x01\n\x16timestampToEventOffset\x12).notification_service.TimeToOffsetRequest\x1a*.notification_service.TimeToOffsetResponse\"A\x82\xd3\xe4\x93\x02;\"6/aiflow/notification-service/timestamp_to_event_offset:\x01*B<\n\x1dorg.aiflow.notification.protoZ\x15/notification_service\x88\x01\x01\x90\x01\x01\x62\x06proto3'
  ,
  dependencies=[google_dot_api_dot_annotations__pb2.DESCRIPTOR,])

_RETURNSTATUS = _descriptor.EnumDescriptor(
  name='ReturnStatus',
  full_name='notification_service.ReturnStatus',
  filename=None,
  file=DESCRIPTOR,
  create_key=_descriptor._internal_create_key,
  values=[
    _descriptor.EnumValueDescriptor(
      name='SUCCESS', index=0, number=0,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='ERROR', index=1, number=1,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=2344,
  serialized_end=2382,
)
_sym_db.RegisterEnumDescriptor(_RETURNSTATUS)

ReturnStatus = enum_type_wrapper.EnumTypeWrapper(_RETURNSTATUS)
SUCCESS = 0
ERROR = 1



_EVENTPROTO = _descriptor.Descriptor(
  name='EventProto',
  full_name='notification_service.EventProto',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='key', full_name='notification_service.EventProto.key', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='value', full_name='notification_service.EventProto.value', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='context', full_name='notification_service.EventProto.context', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='namespace', full_name='notification_service.EventProto.namespace', index=3,
      number=4, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='offset', full_name='notification_service.EventProto.offset', index=4,
      number=5, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='create_time', full_name='notification_service.EventProto.create_time', index=5,
      number=6, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='sender', full_name='notification_service.EventProto.sender', index=6,
      number=7, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=83,
  serialized_end=212,
)


_SENDEREVENTCOUNTPROTO = _descriptor.Descriptor(
  name='SenderEventCountProto',
  full_name='notification_service.SenderEventCountProto',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='sender', full_name='notification_service.SenderEventCountProto.sender', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='event_count', full_name='notification_service.SenderEventCountProto.event_count', index=1,
      number=2, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=214,
  serialized_end=274,
)


_SENDEVENTREQUEST = _descriptor.Descriptor(
  name='SendEventRequest',
  full_name='notification_service.SendEventRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='event', full_name='notification_service.SendEventRequest.event', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='uuid', full_name='notification_service.SendEventRequest.uuid', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='enable_idempotence', full_name='notification_service.SendEventRequest.enable_idempotence', index=2,
      number=3, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=276,
  serialized_end=385,
)


_CLIENTMETA = _descriptor.Descriptor(
  name='ClientMeta',
  full_name='notification_service.ClientMeta',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='namespace', full_name='notification_service.ClientMeta.namespace', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='sender', full_name='notification_service.ClientMeta.sender', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=387,
  serialized_end=434,
)


_SENDEVENTSRESPONSE = _descriptor.Descriptor(
  name='SendEventsResponse',
  full_name='notification_service.SendEventsResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='event', full_name='notification_service.SendEventsResponse.event', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='return_code', full_name='notification_service.SendEventsResponse.return_code', index=1,
      number=2, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='return_msg', full_name='notification_service.SendEventsResponse.return_msg', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=437,
  serialized_end=583,
)


_LISTEVENTSREQUEST = _descriptor.Descriptor(
  name='ListEventsRequest',
  full_name='notification_service.ListEventsRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='key', full_name='notification_service.ListEventsRequest.key', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='namespace', full_name='notification_service.ListEventsRequest.namespace', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='sender', full_name='notification_service.ListEventsRequest.sender', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='start_offset', full_name='notification_service.ListEventsRequest.start_offset', index=3,
      number=4, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='end_offset', full_name='notification_service.ListEventsRequest.end_offset', index=4,
      number=5, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='timeout_seconds', full_name='notification_service.ListEventsRequest.timeout_seconds', index=5,
      number=6, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=586,
  serialized_end=720,
)


_COUNTEVENTSREQUEST = _descriptor.Descriptor(
  name='CountEventsRequest',
  full_name='notification_service.CountEventsRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='key', full_name='notification_service.CountEventsRequest.key', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='namespace', full_name='notification_service.CountEventsRequest.namespace', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='sender', full_name='notification_service.CountEventsRequest.sender', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='start_offset', full_name='notification_service.CountEventsRequest.start_offset', index=3,
      number=4, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='end_offset', full_name='notification_service.CountEventsRequest.end_offset', index=4,
      number=5, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=722,
  serialized_end=832,
)


_LISTALLEVENTSREQUEST = _descriptor.Descriptor(
  name='ListAllEventsRequest',
  full_name='notification_service.ListAllEventsRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='timeout_seconds', full_name='notification_service.ListAllEventsRequest.timeout_seconds', index=0,
      number=1, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='start_time', full_name='notification_service.ListAllEventsRequest.start_time', index=1,
      number=2, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='start_offset', full_name='notification_service.ListAllEventsRequest.start_offset', index=2,
      number=3, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='end_offset', full_name='notification_service.ListAllEventsRequest.end_offset', index=3,
      number=4, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=834,
  serialized_end=943,
)


_LISTEVENTSRESPONSE = _descriptor.Descriptor(
  name='ListEventsResponse',
  full_name='notification_service.ListEventsResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='return_code', full_name='notification_service.ListEventsResponse.return_code', index=0,
      number=1, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='return_msg', full_name='notification_service.ListEventsResponse.return_msg', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='events', full_name='notification_service.ListEventsResponse.events', index=2,
      number=3, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=946,
  serialized_end=1093,
)


_COUNTEVENTSRESPONSE = _descriptor.Descriptor(
  name='CountEventsResponse',
  full_name='notification_service.CountEventsResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='return_code', full_name='notification_service.CountEventsResponse.return_code', index=0,
      number=1, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='return_msg', full_name='notification_service.CountEventsResponse.return_msg', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='event_count', full_name='notification_service.CountEventsResponse.event_count', index=2,
      number=3, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='sender_event_counts', full_name='notification_service.CountEventsResponse.sender_event_counts', index=3,
      number=4, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1096,
  serialized_end=1289,
)


_MEMBERPROTO = _descriptor.Descriptor(
  name='MemberProto',
  full_name='notification_service.MemberProto',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='version', full_name='notification_service.MemberProto.version', index=0,
      number=1, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='server_uri', full_name='notification_service.MemberProto.server_uri', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='update_time', full_name='notification_service.MemberProto.update_time', index=2,
      number=3, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1291,
  serialized_end=1362,
)


_NOTIFY = _descriptor.Descriptor(
  name='Notify',
  full_name='notification_service.Notify',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='key', full_name='notification_service.Notify.key', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='namespace', full_name='notification_service.Notify.namespace', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1364,
  serialized_end=1404,
)


_NOTIFYREQUEST = _descriptor.Descriptor(
  name='NotifyRequest',
  full_name='notification_service.NotifyRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='notifies', full_name='notification_service.NotifyRequest.notifies', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1406,
  serialized_end=1469,
)


_LISTMEMBERSREQUEST = _descriptor.Descriptor(
  name='ListMembersRequest',
  full_name='notification_service.ListMembersRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='timeout_seconds', full_name='notification_service.ListMembersRequest.timeout_seconds', index=0,
      number=1, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1471,
  serialized_end=1516,
)


_LISTMEMBERSRESPONSE = _descriptor.Descriptor(
  name='ListMembersResponse',
  full_name='notification_service.ListMembersResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='return_code', full_name='notification_service.ListMembersResponse.return_code', index=0,
      number=1, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='return_msg', full_name='notification_service.ListMembersResponse.return_msg', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='members', full_name='notification_service.ListMembersResponse.members', index=2,
      number=3, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1519,
  serialized_end=1669,
)


_NOTIFYNEWMEMBERREQUEST = _descriptor.Descriptor(
  name='NotifyNewMemberRequest',
  full_name='notification_service.NotifyNewMemberRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='member', full_name='notification_service.NotifyNewMemberRequest.member', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1671,
  serialized_end=1746,
)


_REGISTERCLIENTREQUEST = _descriptor.Descriptor(
  name='RegisterClientRequest',
  full_name='notification_service.RegisterClientRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='client_meta', full_name='notification_service.RegisterClientRequest.client_meta', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1748,
  serialized_end=1826,
)


_REGISTERCLIENTRESPONSE = _descriptor.Descriptor(
  name='RegisterClientResponse',
  full_name='notification_service.RegisterClientResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='return_code', full_name='notification_service.RegisterClientResponse.return_code', index=0,
      number=1, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='return_msg', full_name='notification_service.RegisterClientResponse.return_msg', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='client_id', full_name='notification_service.RegisterClientResponse.client_id', index=2,
      number=3, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1828,
  serialized_end=1948,
)


_CLIENTIDREQUEST = _descriptor.Descriptor(
  name='ClientIdRequest',
  full_name='notification_service.ClientIdRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='client_id', full_name='notification_service.ClientIdRequest.client_id', index=0,
      number=1, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1950,
  serialized_end=1986,
)


_COMMONRESPONSE = _descriptor.Descriptor(
  name='CommonResponse',
  full_name='notification_service.CommonResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='return_code', full_name='notification_service.CommonResponse.return_code', index=0,
      number=1, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='return_msg', full_name='notification_service.CommonResponse.return_msg', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1988,
  serialized_end=2081,
)


_ISCLIENTEXISTSRESPONSE = _descriptor.Descriptor(
  name='isClientExistsResponse',
  full_name='notification_service.isClientExistsResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='return_code', full_name='notification_service.isClientExistsResponse.return_code', index=0,
      number=1, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='return_msg', full_name='notification_service.isClientExistsResponse.return_msg', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='is_exists', full_name='notification_service.isClientExistsResponse.is_exists', index=2,
      number=3, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2083,
  serialized_end=2203,
)


_TIMETOOFFSETREQUEST = _descriptor.Descriptor(
  name='TimeToOffsetRequest',
  full_name='notification_service.TimeToOffsetRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='timestamp', full_name='notification_service.TimeToOffsetRequest.timestamp', index=0,
      number=1, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2205,
  serialized_end=2245,
)


_TIMETOOFFSETRESPONSE = _descriptor.Descriptor(
  name='TimeToOffsetResponse',
  full_name='notification_service.TimeToOffsetResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='return_code', full_name='notification_service.TimeToOffsetResponse.return_code', index=0,
      number=1, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='offset', full_name='notification_service.TimeToOffsetResponse.offset', index=1,
      number=2, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2247,
  serialized_end=2342,
)

_SENDEVENTREQUEST.fields_by_name['event'].message_type = _EVENTPROTO
_SENDEVENTSRESPONSE.fields_by_name['event'].message_type = _EVENTPROTO
_SENDEVENTSRESPONSE.fields_by_name['return_code'].enum_type = _RETURNSTATUS
_LISTEVENTSRESPONSE.fields_by_name['return_code'].enum_type = _RETURNSTATUS
_LISTEVENTSRESPONSE.fields_by_name['events'].message_type = _EVENTPROTO
_COUNTEVENTSRESPONSE.fields_by_name['return_code'].enum_type = _RETURNSTATUS
_COUNTEVENTSRESPONSE.fields_by_name['sender_event_counts'].message_type = _SENDEREVENTCOUNTPROTO
_NOTIFYREQUEST.fields_by_name['notifies'].message_type = _NOTIFY
_LISTMEMBERSRESPONSE.fields_by_name['return_code'].enum_type = _RETURNSTATUS
_LISTMEMBERSRESPONSE.fields_by_name['members'].message_type = _MEMBERPROTO
_NOTIFYNEWMEMBERREQUEST.fields_by_name['member'].message_type = _MEMBERPROTO
_REGISTERCLIENTREQUEST.fields_by_name['client_meta'].message_type = _CLIENTMETA
_REGISTERCLIENTRESPONSE.fields_by_name['return_code'].enum_type = _RETURNSTATUS
_COMMONRESPONSE.fields_by_name['return_code'].enum_type = _RETURNSTATUS
_ISCLIENTEXISTSRESPONSE.fields_by_name['return_code'].enum_type = _RETURNSTATUS
_TIMETOOFFSETRESPONSE.fields_by_name['return_code'].enum_type = _RETURNSTATUS
DESCRIPTOR.message_types_by_name['EventProto'] = _EVENTPROTO
DESCRIPTOR.message_types_by_name['SenderEventCountProto'] = _SENDEREVENTCOUNTPROTO
DESCRIPTOR.message_types_by_name['SendEventRequest'] = _SENDEVENTREQUEST
DESCRIPTOR.message_types_by_name['ClientMeta'] = _CLIENTMETA
DESCRIPTOR.message_types_by_name['SendEventsResponse'] = _SENDEVENTSRESPONSE
DESCRIPTOR.message_types_by_name['ListEventsRequest'] = _LISTEVENTSREQUEST
DESCRIPTOR.message_types_by_name['CountEventsRequest'] = _COUNTEVENTSREQUEST
DESCRIPTOR.message_types_by_name['ListAllEventsRequest'] = _LISTALLEVENTSREQUEST
DESCRIPTOR.message_types_by_name['ListEventsResponse'] = _LISTEVENTSRESPONSE
DESCRIPTOR.message_types_by_name['CountEventsResponse'] = _COUNTEVENTSRESPONSE
DESCRIPTOR.message_types_by_name['MemberProto'] = _MEMBERPROTO
DESCRIPTOR.message_types_by_name['Notify'] = _NOTIFY
DESCRIPTOR.message_types_by_name['NotifyRequest'] = _NOTIFYREQUEST
DESCRIPTOR.message_types_by_name['ListMembersRequest'] = _LISTMEMBERSREQUEST
DESCRIPTOR.message_types_by_name['ListMembersResponse'] = _LISTMEMBERSRESPONSE
DESCRIPTOR.message_types_by_name['NotifyNewMemberRequest'] = _NOTIFYNEWMEMBERREQUEST
DESCRIPTOR.message_types_by_name['RegisterClientRequest'] = _REGISTERCLIENTREQUEST
DESCRIPTOR.message_types_by_name['RegisterClientResponse'] = _REGISTERCLIENTRESPONSE
DESCRIPTOR.message_types_by_name['ClientIdRequest'] = _CLIENTIDREQUEST
DESCRIPTOR.message_types_by_name['CommonResponse'] = _COMMONRESPONSE
DESCRIPTOR.message_types_by_name['isClientExistsResponse'] = _ISCLIENTEXISTSRESPONSE
DESCRIPTOR.message_types_by_name['TimeToOffsetRequest'] = _TIMETOOFFSETREQUEST
DESCRIPTOR.message_types_by_name['TimeToOffsetResponse'] = _TIMETOOFFSETRESPONSE
DESCRIPTOR.enum_types_by_name['ReturnStatus'] = _RETURNSTATUS
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

EventProto = _reflection.GeneratedProtocolMessageType('EventProto', (_message.Message,), {
  'DESCRIPTOR' : _EVENTPROTO,
  '__module__' : 'notification_service_pb2'
  # @@protoc_insertion_point(class_scope:notification_service.EventProto)
  })
_sym_db.RegisterMessage(EventProto)

SenderEventCountProto = _reflection.GeneratedProtocolMessageType('SenderEventCountProto', (_message.Message,), {
  'DESCRIPTOR' : _SENDEREVENTCOUNTPROTO,
  '__module__' : 'notification_service_pb2'
  # @@protoc_insertion_point(class_scope:notification_service.SenderEventCountProto)
  })
_sym_db.RegisterMessage(SenderEventCountProto)

SendEventRequest = _reflection.GeneratedProtocolMessageType('SendEventRequest', (_message.Message,), {
  'DESCRIPTOR' : _SENDEVENTREQUEST,
  '__module__' : 'notification_service_pb2'
  # @@protoc_insertion_point(class_scope:notification_service.SendEventRequest)
  })
_sym_db.RegisterMessage(SendEventRequest)

ClientMeta = _reflection.GeneratedProtocolMessageType('ClientMeta', (_message.Message,), {
  'DESCRIPTOR' : _CLIENTMETA,
  '__module__' : 'notification_service_pb2'
  # @@protoc_insertion_point(class_scope:notification_service.ClientMeta)
  })
_sym_db.RegisterMessage(ClientMeta)

SendEventsResponse = _reflection.GeneratedProtocolMessageType('SendEventsResponse', (_message.Message,), {
  'DESCRIPTOR' : _SENDEVENTSRESPONSE,
  '__module__' : 'notification_service_pb2'
  # @@protoc_insertion_point(class_scope:notification_service.SendEventsResponse)
  })
_sym_db.RegisterMessage(SendEventsResponse)

ListEventsRequest = _reflection.GeneratedProtocolMessageType('ListEventsRequest', (_message.Message,), {
  'DESCRIPTOR' : _LISTEVENTSREQUEST,
  '__module__' : 'notification_service_pb2'
  # @@protoc_insertion_point(class_scope:notification_service.ListEventsRequest)
  })
_sym_db.RegisterMessage(ListEventsRequest)

CountEventsRequest = _reflection.GeneratedProtocolMessageType('CountEventsRequest', (_message.Message,), {
  'DESCRIPTOR' : _COUNTEVENTSREQUEST,
  '__module__' : 'notification_service_pb2'
  # @@protoc_insertion_point(class_scope:notification_service.CountEventsRequest)
  })
_sym_db.RegisterMessage(CountEventsRequest)

ListAllEventsRequest = _reflection.GeneratedProtocolMessageType('ListAllEventsRequest', (_message.Message,), {
  'DESCRIPTOR' : _LISTALLEVENTSREQUEST,
  '__module__' : 'notification_service_pb2'
  # @@protoc_insertion_point(class_scope:notification_service.ListAllEventsRequest)
  })
_sym_db.RegisterMessage(ListAllEventsRequest)

ListEventsResponse = _reflection.GeneratedProtocolMessageType('ListEventsResponse', (_message.Message,), {
  'DESCRIPTOR' : _LISTEVENTSRESPONSE,
  '__module__' : 'notification_service_pb2'
  # @@protoc_insertion_point(class_scope:notification_service.ListEventsResponse)
  })
_sym_db.RegisterMessage(ListEventsResponse)

CountEventsResponse = _reflection.GeneratedProtocolMessageType('CountEventsResponse', (_message.Message,), {
  'DESCRIPTOR' : _COUNTEVENTSRESPONSE,
  '__module__' : 'notification_service_pb2'
  # @@protoc_insertion_point(class_scope:notification_service.CountEventsResponse)
  })
_sym_db.RegisterMessage(CountEventsResponse)

MemberProto = _reflection.GeneratedProtocolMessageType('MemberProto', (_message.Message,), {
  'DESCRIPTOR' : _MEMBERPROTO,
  '__module__' : 'notification_service_pb2'
  # @@protoc_insertion_point(class_scope:notification_service.MemberProto)
  })
_sym_db.RegisterMessage(MemberProto)

Notify = _reflection.GeneratedProtocolMessageType('Notify', (_message.Message,), {
  'DESCRIPTOR' : _NOTIFY,
  '__module__' : 'notification_service_pb2'
  # @@protoc_insertion_point(class_scope:notification_service.Notify)
  })
_sym_db.RegisterMessage(Notify)

NotifyRequest = _reflection.GeneratedProtocolMessageType('NotifyRequest', (_message.Message,), {
  'DESCRIPTOR' : _NOTIFYREQUEST,
  '__module__' : 'notification_service_pb2'
  # @@protoc_insertion_point(class_scope:notification_service.NotifyRequest)
  })
_sym_db.RegisterMessage(NotifyRequest)

ListMembersRequest = _reflection.GeneratedProtocolMessageType('ListMembersRequest', (_message.Message,), {
  'DESCRIPTOR' : _LISTMEMBERSREQUEST,
  '__module__' : 'notification_service_pb2'
  # @@protoc_insertion_point(class_scope:notification_service.ListMembersRequest)
  })
_sym_db.RegisterMessage(ListMembersRequest)

ListMembersResponse = _reflection.GeneratedProtocolMessageType('ListMembersResponse', (_message.Message,), {
  'DESCRIPTOR' : _LISTMEMBERSRESPONSE,
  '__module__' : 'notification_service_pb2'
  # @@protoc_insertion_point(class_scope:notification_service.ListMembersResponse)
  })
_sym_db.RegisterMessage(ListMembersResponse)

NotifyNewMemberRequest = _reflection.GeneratedProtocolMessageType('NotifyNewMemberRequest', (_message.Message,), {
  'DESCRIPTOR' : _NOTIFYNEWMEMBERREQUEST,
  '__module__' : 'notification_service_pb2'
  # @@protoc_insertion_point(class_scope:notification_service.NotifyNewMemberRequest)
  })
_sym_db.RegisterMessage(NotifyNewMemberRequest)

RegisterClientRequest = _reflection.GeneratedProtocolMessageType('RegisterClientRequest', (_message.Message,), {
  'DESCRIPTOR' : _REGISTERCLIENTREQUEST,
  '__module__' : 'notification_service_pb2'
  # @@protoc_insertion_point(class_scope:notification_service.RegisterClientRequest)
  })
_sym_db.RegisterMessage(RegisterClientRequest)

RegisterClientResponse = _reflection.GeneratedProtocolMessageType('RegisterClientResponse', (_message.Message,), {
  'DESCRIPTOR' : _REGISTERCLIENTRESPONSE,
  '__module__' : 'notification_service_pb2'
  # @@protoc_insertion_point(class_scope:notification_service.RegisterClientResponse)
  })
_sym_db.RegisterMessage(RegisterClientResponse)

ClientIdRequest = _reflection.GeneratedProtocolMessageType('ClientIdRequest', (_message.Message,), {
  'DESCRIPTOR' : _CLIENTIDREQUEST,
  '__module__' : 'notification_service_pb2'
  # @@protoc_insertion_point(class_scope:notification_service.ClientIdRequest)
  })
_sym_db.RegisterMessage(ClientIdRequest)

CommonResponse = _reflection.GeneratedProtocolMessageType('CommonResponse', (_message.Message,), {
  'DESCRIPTOR' : _COMMONRESPONSE,
  '__module__' : 'notification_service_pb2'
  # @@protoc_insertion_point(class_scope:notification_service.CommonResponse)
  })
_sym_db.RegisterMessage(CommonResponse)

isClientExistsResponse = _reflection.GeneratedProtocolMessageType('isClientExistsResponse', (_message.Message,), {
  'DESCRIPTOR' : _ISCLIENTEXISTSRESPONSE,
  '__module__' : 'notification_service_pb2'
  # @@protoc_insertion_point(class_scope:notification_service.isClientExistsResponse)
  })
_sym_db.RegisterMessage(isClientExistsResponse)

TimeToOffsetRequest = _reflection.GeneratedProtocolMessageType('TimeToOffsetRequest', (_message.Message,), {
  'DESCRIPTOR' : _TIMETOOFFSETREQUEST,
  '__module__' : 'notification_service_pb2'
  # @@protoc_insertion_point(class_scope:notification_service.TimeToOffsetRequest)
  })
_sym_db.RegisterMessage(TimeToOffsetRequest)

TimeToOffsetResponse = _reflection.GeneratedProtocolMessageType('TimeToOffsetResponse', (_message.Message,), {
  'DESCRIPTOR' : _TIMETOOFFSETRESPONSE,
  '__module__' : 'notification_service_pb2'
  # @@protoc_insertion_point(class_scope:notification_service.TimeToOffsetResponse)
  })
_sym_db.RegisterMessage(TimeToOffsetResponse)


DESCRIPTOR._options = None

_NOTIFICATIONSERVICE = _descriptor.ServiceDescriptor(
  name='NotificationService',
  full_name='notification_service.NotificationService',
  file=DESCRIPTOR,
  index=0,
  serialized_options=None,
  create_key=_descriptor._internal_create_key,
  serialized_start=2385,
  serialized_end=4120,
  methods=[
  _descriptor.MethodDescriptor(
    name='sendEvent',
    full_name='notification_service.NotificationService.sendEvent',
    index=0,
    containing_service=None,
    input_type=_SENDEVENTREQUEST,
    output_type=_SENDEVENTSRESPONSE,
    serialized_options=b'\202\323\344\223\002,\"\'/aiflow/notification-service/send_event:\001*',
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='listEvents',
    full_name='notification_service.NotificationService.listEvents',
    index=1,
    containing_service=None,
    input_type=_LISTEVENTSREQUEST,
    output_type=_LISTEVENTSRESPONSE,
    serialized_options=b'\202\323\344\223\002*\022(/aiflow/notification-service/list_events',
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='countEvents',
    full_name='notification_service.NotificationService.countEvents',
    index=2,
    containing_service=None,
    input_type=_COUNTEVENTSREQUEST,
    output_type=_COUNTEVENTSRESPONSE,
    serialized_options=b'\202\323\344\223\002+\022)/aiflow/notification-service/count_events',
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='listAllEvents',
    full_name='notification_service.NotificationService.listAllEvents',
    index=3,
    containing_service=None,
    input_type=_LISTALLEVENTSREQUEST,
    output_type=_LISTEVENTSRESPONSE,
    serialized_options=b'\202\323\344\223\002.\022,/aiflow/notification-service/list_all_events',
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='notify',
    full_name='notification_service.NotificationService.notify',
    index=4,
    containing_service=None,
    input_type=_NOTIFYREQUEST,
    output_type=_COMMONRESPONSE,
    serialized_options=b'\202\323\344\223\002(\"#/aiflow/notification-service/notify:\001*',
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='listMembers',
    full_name='notification_service.NotificationService.listMembers',
    index=5,
    containing_service=None,
    input_type=_LISTMEMBERSREQUEST,
    output_type=_LISTMEMBERSRESPONSE,
    serialized_options=b'\202\323\344\223\002+\022)/aiflow/notification-service/list_members',
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='notifyNewMember',
    full_name='notification_service.NotificationService.notifyNewMember',
    index=6,
    containing_service=None,
    input_type=_NOTIFYNEWMEMBERREQUEST,
    output_type=_COMMONRESPONSE,
    serialized_options=b'\202\323\344\223\0023\"./aiflow/notification-service/notify_new_member:\001*',
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='registerClient',
    full_name='notification_service.NotificationService.registerClient',
    index=7,
    containing_service=None,
    input_type=_REGISTERCLIENTREQUEST,
    output_type=_REGISTERCLIENTRESPONSE,
    serialized_options=b'\202\323\344\223\0021\",/aiflow/notification-service/register_client:\001*',
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='deleteClient',
    full_name='notification_service.NotificationService.deleteClient',
    index=8,
    containing_service=None,
    input_type=_CLIENTIDREQUEST,
    output_type=_COMMONRESPONSE,
    serialized_options=b'\202\323\344\223\002/\"*/aiflow/notification-service/delete_client:\001*',
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='isClientExists',
    full_name='notification_service.NotificationService.isClientExists',
    index=9,
    containing_service=None,
    input_type=_CLIENTIDREQUEST,
    output_type=_ISCLIENTEXISTSRESPONSE,
    serialized_options=b'\202\323\344\223\0022\"-/aiflow/notification-service/is_client_exists:\001*',
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='timestampToEventOffset',
    full_name='notification_service.NotificationService.timestampToEventOffset',
    index=10,
    containing_service=None,
    input_type=_TIMETOOFFSETREQUEST,
    output_type=_TIMETOOFFSETRESPONSE,
    serialized_options=b'\202\323\344\223\002;\"6/aiflow/notification-service/timestamp_to_event_offset:\001*',
    create_key=_descriptor._internal_create_key,
  ),
])
_sym_db.RegisterServiceDescriptor(_NOTIFICATIONSERVICE)

DESCRIPTOR.services_by_name['NotificationService'] = _NOTIFICATIONSERVICE

NotificationService = service_reflection.GeneratedServiceType('NotificationService', (_service.Service,), dict(
  DESCRIPTOR = _NOTIFICATIONSERVICE,
  __module__ = 'notification_service_pb2'
  ))

NotificationService_Stub = service_reflection.GeneratedServiceStubType('NotificationService_Stub', (NotificationService,), dict(
  DESCRIPTOR = _NOTIFICATIONSERVICE,
  __module__ = 'notification_service_pb2'
  ))


# @@protoc_insertion_point(module_scope)
